@@ -5,7 +5,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2011	PgPool Global Development Group
+ * Copyright (c) 2003-2013	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -68,7 +68,7 @@ int pool_do_auth(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *cp)
 	int i;
 	StartupPacket *sp;
 
-	protoMajor = MAJOR(cp);
+	protoMajor = MASTER_CONNECTION(cp)->sp->major;
 
 	kind = pool_read_kind(cp);
 	if (kind < 0)
@@ -330,12 +330,6 @@ from pool_read_message_length and recheck the pg_hba.conf settings.");
 	 */
 	if (protoMajor == PROTO_MAJOR_V3)
 	{
-		if (kind != 'K')
-		{
-			pool_error("pool_do_auth: expect \"K\" got %c", kind);
-			return -1;
-		}
-
 		if ((length = pool_read_message_length(cp)) != 12)
 		{
 			pool_error("pool_do_auth: invalid messages length(%d) for BackendKeyData", length);
@@ -347,6 +341,7 @@ from pool_read_message_length and recheck the pg_hba.conf settings.");
 	 * OK, read pid and secret key
 	 */
 	sp = MASTER_CONNECTION(cp)->sp;
+	pid = -1;
 
 	for (i=0;i<NUM_BACKENDS;i++)
 	{
@@ -379,14 +374,12 @@ from pool_read_message_length and recheck the pg_hba.conf settings.");
 		}
 	}
 
-#ifdef NOT_USED
-	sp = MASTER_CONNECTION(cp)->sp;
-	cp->info->major = sp->major;
-	cp->info->minor = sp->minor;
-	strncpy(cp->info->database, sp->database, sizeof(cp->info->database) - 1);
-	strncpy(cp->info->user, sp->user, sizeof(cp->info->user) - 1);
-	cp->info->counter = 1;
-#endif
+	if (pid == -1)
+	{
+		pool_error("pool_do_auth: all backends are down");
+		return -1;
+	}
+
 	return pool_send_backend_key_data(frontend, pid, key, protoMajor);
 }
 
@@ -1047,17 +1040,17 @@ static int read_password_packet(POOL_CONNECTION *frontend, int protoMajor, 	char
 
 		if (pool_read(frontend, &k, sizeof(k)))
 		{
-			pool_debug("read_password_packet_password: failed to read password packet \"p\"");
+			pool_debug("read_password_packet: failed to read password packet \"p\"");
 			return -1;
 		}
 		if (k != 'p')
 		{
-			pool_error("read_password_packet_password: password packet does not start with \"p\"");
+			pool_error("read_password_packet: password packet does not start with \"p\"");
 			return -1;
 		}
 		if (pool_read(frontend, &size, sizeof(size)))
 		{
-			pool_error("read_password_packet_password: failed to read password packet size");
+			pool_error("read_password_packet: failed to read password packet size");
 			return -1;
 		}
 	}
@@ -1282,6 +1275,7 @@ signed char pool_read_kind(POOL_CONNECTION_POOL *cp)
 	char kind0, kind;
 	int i;
 
+	kind = -1;
 	kind0 = 0;
 
 	for (i=0;i<NUM_BACKENDS;i++)
@@ -1338,6 +1332,7 @@ int pool_read_int(POOL_CONNECTION_POOL *cp)
 	int data0, data;
 	int i;
 
+	data = -1;
 	data0 = 0;
 
 	for (i=0;i<NUM_BACKENDS;i++)
